@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,13 +23,18 @@
 #include "zbxalgo.h"
 #include "vectorimpl.h"
 
-ZBX_VECTOR_IMPL(uint64, zbx_uint64_t);
-ZBX_PTR_VECTOR_IMPL(str, char *);
-ZBX_PTR_VECTOR_IMPL(ptr, void *);
-ZBX_VECTOR_IMPL(ptr_pair, zbx_ptr_pair_t);
-ZBX_VECTOR_IMPL(uint64_pair, zbx_uint64_pair_t);
+ZBX_VECTOR_IMPL(uint64, zbx_uint64_t)
+ZBX_PTR_VECTOR_IMPL(str, char *)
+ZBX_PTR_VECTOR_IMPL(ptr, void *)
+ZBX_VECTOR_IMPL(ptr_pair, zbx_ptr_pair_t)
+ZBX_VECTOR_IMPL(uint64_pair, zbx_uint64_pair_t)
 
 void	zbx_ptr_free(void *data)
 {
 	zbx_free(data);
 }
+
+void	zbx_str_free(char *data)
+{
+	zbx_free(data);
+}
